#brailleDisplayDrivers/syncbraille.py
#A part of NonVisual Desktop Access (NVDA)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#Copyright (C) 2010 Gianluca Casalino <gianluca@spazioausili.net>

from ctypes import *
from ctypes.wintypes import *
import braille
import queueHandler
from logHandler import log
from keyboardHandler import KeyboardInputGesture

pressedKeys=set()
releasedKeys=set()
SYNC_KEYPRESSED = 0x01
SYNC_KEYRELEASED = 0x02
SYNC_CURSORROUTING = 0x00
LRESULT=c_long
HCURSOR=c_long

#MAP OF KEYS

SCROLL_LEFT = 8192
SCROLL_RIGHT = 16384

try:
	syncBrlLib = cdll.LoadLibrary("brailleDisplayDrivers\\SyncBraille.dll")
except:
	syncBrlLib = None

#Standard window class stuff

WNDPROC=WINFUNCTYPE(LRESULT,HWND,c_uint,WPARAM,LPARAM)

class WNDCLASSEXW(Structure):
	_fields_=[
		('cbSize',c_uint),
		('style',c_uint),
		('lpfnWndProc',WNDPROC),
		('cbClsExtra',c_int),
		('cbWndExtra',c_int),
		('hInstance',HINSTANCE),
		('hIcon',HICON),
		('HCURSOR',HCURSOR),
		('hbrBackground',HBRUSH),
		('lpszMenuName',LPWSTR),
		('lpszClassName',LPWSTR),
		('hIconSm',HICON),
	]

appInstance=windll.kernel32.GetModuleHandleW(None)

nvdaBsBrlWm=windll.user32.RegisterWindowMessageW(u"nvdaBsBrlWm")

@WNDPROC
def nvdaBsBrlWndProc(hwnd,msg,wParam,lParam):
	global pressedKeys, releasedKeys
	if msg == nvdaBsBrlWm and wParam==SYNC_KEYPRESSED:
		pressedKeys.add(lParam)
		if lParam == SCROLL_LEFT: 
			queueHandler.queueFunction(queueHandler.eventQueue,braille.handler.scrollBack)
		elif lParam == SCROLL_RIGHT:
			queueHandler.queueFunction(queueHandler.eventQueue,braille.handler.scrollForward)
	if msg == nvdaBsBrlWm and wParam==SYNC_KEYRELEASED:
		releasedKeys.add(lParam)
		if pressedKeys==releasedKeys: 
			_handleKeys(pressedKeys)
			pressedKeys.clear()
			releasedKeys.clear()
	elif msg == nvdaBsBrlWm and wParam == SYNC_CURSORROUTING:
		queueHandler.queueFunction(queueHandler.eventQueue,braille.handler.routeTo, lParam)
	return windll.user32.DefWindowProcW(hwnd,msg,wParam,lParam)

def _handleKeys(keys):
		pass

nvdaBsBrlWndCls=WNDCLASSEXW()
nvdaBsBrlWndCls.cbSize=sizeof(nvdaBsBrlWndCls)
nvdaBsBrlWndCls.lpfnWndProc=nvdaBsBrlWndProc
nvdaBsBrlWndCls.hInstance=appInstance
nvdaBsBrlWndCls.lpszClassName=u"nvdaBsBrlWndCls"

class BrailleDisplayDriver(braille.BrailleDisplayDriverWithCursor):
	"""SyncBraille.
	"""
	name = "syncbraille"
	description = "SyncBraille"

	@classmethod
	def check(cls):
		return bool(syncBrlLib)

	def __init__(self):
		super(BrailleDisplayDriver, self).__init__()
		self._messageWindowClassAtom=windll.user32.RegisterClassExW(byref(nvdaBsBrlWndCls))
		self._messageWindow=windll.user32.CreateWindowExW(0,self._messageWindowClassAtom,u"nvdaBsBrlWndCls window",0,0,0,0,0,None,None,appInstance,None)
		connected = syncBrlLib.OpenSyncBrl(self._messageWindow,nvdaBsBrlWm)
		if connected>=1: return
		raise RuntimeError("No display found")
 
	def terminate(self):
		super(BrailleDisplayDriver, self).terminate()
		syncBrlLib.CloseSyncBrl()
		windll.user32.DestroyWindow(self._messageWindow)
		windll.user32.UnregisterClassW(self._messageWindowClassAtom,appInstance)

 	def _get_numCells(self):
		return syncBrlLib.GetCellCount()

	def _display(self, cells):
		cells = "".join([chr(x) for x in cells])
		syncBrlLib.SendSyncBrl(cells)
